﻿using System;
using PvrSharp.Helpers;

namespace PvrSharp.Formats
{
	/// <summary>
	/// http://www.khronos.org/registry/gles/extensions/OES/OES_compressed_ETC1_RGB8_texture.txt
	/// </summary>
	public class Etc1 : IColorFormat
	{
		private static Etc1 instance;

		public static Etc1 Instance
		{
			get { return instance ?? (instance = new Etc1()); }
		}

		public bool HasAlpha { get { return false; } }

		private Etc1()
		{
		}

		public byte[] ConvertToRgb888(PvrFile file, int sourceOffset)
		{
			var result = new byte[file.Width * file.Height * 3];
			int destinationIndex = 0;
			int sourceIndex = sourceOffset;

			uint numberOfWidthBlocks = file.Width / 4;
			uint numberOfHeightBlocks = file.Height / 4;
			for (int rowBlockIndex = 0; rowBlockIndex < numberOfHeightBlocks; rowBlockIndex++)
			{
				for (int colBlockIndex = 0; colBlockIndex < numberOfWidthBlocks; colBlockIndex++)
				{
					byte q0 = file.RawPixelData[sourceIndex++];
					byte q1 = file.RawPixelData[sourceIndex++];
					byte q2 = file.RawPixelData[sourceIndex++];
					byte q3 = file.RawPixelData[sourceIndex++];
					byte q4 = file.RawPixelData[sourceIndex++];
					byte q5 = file.RawPixelData[sourceIndex++];
					byte q6 = file.RawPixelData[sourceIndex++];
					byte q7 = file.RawPixelData[sourceIndex++];
					long allBits = 256 * (256 * (256 * (256 * (256 * (256 * (256 * q0 + q1) + q2) + q3) + q4) + q5) + q6) + q7;
					int diffBit = (int)(allBits >> 32) & 1;
					int flipBit = (int)(allBits >> 31) & 1;

					if (diffBit == 0)
					{
						// individual mode
						int baseColorSub1RCodeword = (int)(allBits >> 59) & BitsHelper.And4;
						int baseColorSub2RCodeword = (int)(allBits >> 55) & BitsHelper.And4;
						int baseColorSub1GCodeword = (int)(allBits >> 51) & BitsHelper.And4;
						int baseColorSub2GCodeword = (int)(allBits >> 47) & BitsHelper.And4;
						int baseColorSub1BCodeword = (int)(allBits >> 43) & BitsHelper.And4;
						int baseColorSub2BCodeword = (int)(allBits >> 39) & BitsHelper.And4;
						byte baseColorSub1R = (byte)((baseColorSub1RCodeword << 4) + baseColorSub1RCodeword);
						byte baseColorSub1G = (byte)((baseColorSub1GCodeword << 4) + baseColorSub1GCodeword);
						byte baseColorSub1B = (byte)((baseColorSub1BCodeword << 4) + baseColorSub1BCodeword);
						byte baseColorSub2R = (byte)((baseColorSub2RCodeword << 4) + baseColorSub2RCodeword);
						byte baseColorSub2G = (byte)((baseColorSub2GCodeword << 4) + baseColorSub2GCodeword);
						byte baseColorSub2B = (byte)((baseColorSub2BCodeword << 4) + baseColorSub2BCodeword);
						result[destinationIndex++] = baseColorSub1R;
						result[destinationIndex++] = baseColorSub1G;
						result[destinationIndex++] = baseColorSub1B;
						result[destinationIndex++] = baseColorSub2R;
						result[destinationIndex++] = baseColorSub2G;
						result[destinationIndex++] = baseColorSub2B;
					}
					else
					{
						// differential mode
					}

					if (flipBit == 0)
					{
						// two 2x4 subblocks side-by-side
					}
					else
					{
						// two 4x2 subblocks on top of each other
					}
				}
			}

			return result;
		}

		public byte[] ConvertToRgb8888(PvrFile file, int sourceOffset)
		{
			throw new NotImplementedException();
		}
	}
}
