﻿using System;
using PvrSharp.Helpers;

namespace PvrSharp.Formats
{
	public class Rgb565 : IColorFormat
	{
		private static Rgb565 instance;

		public static Rgb565 Instance
		{
			get { return instance ?? (instance = new Rgb565()); }
		}

		public bool HasAlpha { get { return false; } }

		private Rgb565()
		{
		}

		public byte[] ConvertToRgb888(PvrFile file, int sourceOffset)
		{
			var result = new byte[file.Width * file.Height * 3];
			int destinationIndex = 0;
			int sourceIndex = sourceOffset;
			for (int rowIndex = 0; rowIndex < file.Height; rowIndex++)
			{
				for (int pixelIndex = 0; pixelIndex < file.Width; pixelIndex++)
				{
					int data = BitsHelper.GetTwoBytes(file.RawPixelData, ref sourceIndex);
					// rgb565
					// |-------| |-----------| |-------|
					// 7 6 5 4 3 2 1 0 | 7 6 5 4 3 2 1 0
					byte r = (byte)(data >> 11);
					byte g = (byte)((data >> 5) & BitsHelper.And6);
					byte b = (byte)(data & BitsHelper.And5);

					result[destinationIndex++] = (byte)(r * BitsHelper.Factor5);
					result[destinationIndex++] = (byte)(g * BitsHelper.Factor6);
					result[destinationIndex++] = (byte)(b * BitsHelper.Factor5);
				}
			}

			return result;
		}

		public byte[] ConvertToRgb8888(PvrFile file, int sourceOffset)
		{
			throw new NotImplementedException();
		}
	}
}
