﻿using PvrSharp.Helpers;

namespace PvrSharp.Formats
{
	public class Rgba4444 : IColorFormat
	{
		private static Rgba4444 instance;

		public static Rgba4444 Instance
		{
			get { return instance ?? (instance = new Rgba4444()); }
		}

		public bool HasAlpha { get { return true; } }

		private Rgba4444()
		{
		}

		public byte[] ConvertToRgb888(PvrFile file, int sourceOffset)
		{
			var result = new byte[file.Width * file.Height * 3];
			int destinationIndex = 0;
			int sourceIndex = sourceOffset;
			for (int rowIndex = 0; rowIndex < file.Height; rowIndex++)
			{
				for (int pixelIndex = 0; pixelIndex < file.Width; pixelIndex++)
				{
					int data = BitsHelper.GetTwoBytes(file.RawPixelData, ref sourceIndex);
					int r = data >> 12;
					int g = (data >> 8) & BitsHelper.And4;
					int b = (data >> 4) & BitsHelper.And4;
					result[destinationIndex++] = (byte)(r * BitsHelper.Factor4);
					result[destinationIndex++] = (byte)(g * BitsHelper.Factor4);
					result[destinationIndex++] = (byte)(b * BitsHelper.Factor4);
				}
			}

			return result;
		}

		public byte[] ConvertToRgb8888(PvrFile file, int sourceOffset)
		{
			var result = new byte[file.Width * file.Height * 4];
			int destinationIndex = 0;
			int sourceIndex = sourceOffset;
			for (int rowIndex = 0; rowIndex < file.Height; rowIndex++)
			{
				for (int pixelIndex = 0; pixelIndex < file.Width; pixelIndex++)
				{
					int data = BitsHelper.GetTwoBytes(file.RawPixelData, ref sourceIndex);
					int r = data >> 12;
					int g = (data >> 8) & BitsHelper.And4;
					int b = (data >> 4) & BitsHelper.And4;
					int a = data & BitsHelper.And4;
					result[destinationIndex++] = (byte)(r * BitsHelper.Factor4);
					result[destinationIndex++] = (byte)(g * BitsHelper.Factor4);
					result[destinationIndex++] = (byte)(b * BitsHelper.Factor4);
					result[destinationIndex++] = (byte)(a * BitsHelper.Factor4);
				}
			}

			return result;
		}
	}
}
