﻿using PvrSharp.Helpers;

namespace PvrSharp.Formats
{
	public class Rgba5551 : IColorFormat
	{
		private static Rgba5551 instance;

		public static Rgba5551 Instance
		{
			get { return instance ?? (instance = new Rgba5551()); }
		}

		public bool HasAlpha { get { return true; } }

		private Rgba5551()
		{
		}

		public byte[] ConvertToRgb888(PvrFile file, int sourceOffset)
		{
			var result = new byte[file.Width * file.Height * 3];
			int destinationIndex = 0;
			int sourceIndex = sourceOffset;
			for (int rowIndex = 0; rowIndex < file.Height; rowIndex++)
			{
				for (int pixelIndex = 0; pixelIndex < file.Width; pixelIndex++)
				{
					int data = BitsHelper.GetTwoBytes(file.RawPixelData, ref sourceIndex);
					int r = data >> 11;
					int g = (data >> 6) & BitsHelper.And5;
					int b = (data >> 1) & BitsHelper.And5;
					result[destinationIndex++] = (byte)(r * BitsHelper.Factor5);
					result[destinationIndex++] = (byte)(g * BitsHelper.Factor5);
					result[destinationIndex++] = (byte)(b * BitsHelper.Factor5);
				}
			}

			return result;
		}

		public byte[] ConvertToRgb8888(PvrFile file, int sourceOffset)
		{
			var result = new byte[file.Width * file.Height * 4];
			int destinationIndex = 0;
			int sourceIndex = sourceOffset;
			for (int rowIndex = 0; rowIndex < file.Height; rowIndex++)
			{
				for (int pixelIndex = 0; pixelIndex < file.Width; pixelIndex++)
				{
					int data = BitsHelper.GetTwoBytes(file.RawPixelData, ref sourceIndex);
					int r = data >> 11;
					int g = (data >> 6) & BitsHelper.And5;
					int b = (data >> 1) & BitsHelper.And5;
					result[destinationIndex++] = (byte)(r * BitsHelper.Factor5);
					result[destinationIndex++] = (byte)(g * BitsHelper.Factor5);
					result[destinationIndex++] = (byte)(b * BitsHelper.Factor5);
					result[destinationIndex++] = (byte)((data & 1) == 1 ? 255 : 0);
				}
			}

			return result;
		}
	}
}
