﻿using System;
using System.IO;

namespace PvrSharp
{
	public class PvrFile
	{
		public byte[] RawPixelData { get; private set; }
		public PvrPixelFormat PixelFormat { get; private set; }
		public PvrMetaDataCollection MetaDataEntries { get; private set; }
		public uint Width { get; private set; }
		public uint Height { get; private set; }
		public uint Depth { get; private set; }
		public uint NumberOfSurfaces { get; private set; }
		public uint NumberOfFaces { get; private set; }
		public uint NumberOfMipMaps { get; private set; }
		public PvrFlags Flags { get; private set; }
		public PvrColorSpace ColorSpace { get; private set; }
		public PvrChannelType ChannelType { get; private set; }

		private PvrFile()
		{
			PixelFormat = new PvrPixelFormat();
		}

		public static PvrFile FromStream(Stream stream)
		{
			var result = new PvrFile();
			var reader = new BinaryReader(stream);
			result.Parse(reader);
			return result;
		}

		private void Parse(BinaryReader reader)
		{
			uint version = reader.ReadUInt32();
			if (version != 0x03525650 && version != 0x50556203)
				throw new Exception("This file has an invalid pvr version in the header!");

			Flags = (PvrFlags)reader.ReadUInt32();
			PixelFormat.Parse(reader);
			ColorSpace = (PvrColorSpace)reader.ReadUInt32();
			ChannelType = (PvrChannelType)reader.ReadUInt32();
			Height = reader.ReadUInt32();
			Width = reader.ReadUInt32();
			Depth = reader.ReadUInt32();
			NumberOfSurfaces = reader.ReadUInt32();
			NumberOfFaces = reader.ReadUInt32();
			NumberOfMipMaps = reader.ReadUInt32();
			MetaDataEntries = PvrMetaDataCollection.FromStream(reader);

			long sizeOfPixelData = reader.BaseStream.Length - reader.BaseStream.Position;
			RawPixelData = reader.ReadBytes((int)sizeOfPixelData);
		}

		public byte[] GetColorData()
		{
			byte[] result = null;
			//for(int mipmapIndex = 0; mipmapIndex < NumberOfMipMaps; mipmapIndex++)
			for (int surfaceIndex = 0; surfaceIndex < NumberOfSurfaces; surfaceIndex++)
				for (int faceIndex = 0; faceIndex < NumberOfFaces; faceIndex++)
					for (int sliceIndex = 0; sliceIndex < Depth; sliceIndex++)
					{
						result = PixelFormat.ColorFormat.HasAlpha ?
							PixelFormat.ColorFormat.ConvertToRgb8888(this, 0) :
							PixelFormat.ColorFormat.ConvertToRgb888(this, 0);
					}

			return result;
		}

		public override string ToString()
		{
			string result = "";
			result += "Width: " + Width + "\n";
			result += "Height: " + Height + "\n";
			result += "Depth: " + Depth + "\n";
			result += "NumberOfSurfaces: " + NumberOfSurfaces + "\n";
			result += "NumberOfFaces: " + NumberOfFaces + "\n";
			result += "NumberOfMipMaps: " + NumberOfMipMaps + "\n";
			result += "PixelFormat: " + PixelFormat + "\n";
			result += "Flags: " + Flags + "\n";
			result += "ColorSpace: " + ColorSpace + "\n";
			result += "ChannelType: " + ChannelType + "\n";
			return result;
		}
	}
}
