﻿using System.IO;
using System.Collections.Generic;
using PvrSharp.Formats;

namespace PvrSharp
{
	public class PvrPixelFormat
	{
		private static readonly Dictionary<string, IColorFormat> ColorLookup = new Dictionary<string, IColorFormat>
		{
			{"la88", La88.Instance},
			{"rgb565", Rgb565.Instance},
			{"rgb888", Rgb888.Instance},
			{"rgba8888", Rgba8888.Instance},
			{"rgba5551", Rgba5551.Instance},
			{"rgba4444", Rgba4444.Instance},
			{"rgba32323232", null},
			{"r8", R8.Instance},

			{"PvrTc2BppRgb", null},
			{"PvrTc2BppRgba", null},
			{"PvrTc4BppRgb", null},
			{"PvrTc4BppRgba", null},
			{"PvrTcII2Bp", null},
			{"PvrTcII4Bpp", null},
			{"Dxt1", null},
			{"Dxt2", null},
			{"Dxt3", null},
			{"Dxt4", null},
			{"Dxt5", null},
			{"Bc1", null},
			{"Bc2", null},
			{"Bc3", null},
			{"Bc4", null},
			{"Bc5", null},
			{"Bc6", null},
			{"Bc7", null},
			{"UYVY", null},
			{"YUY2", null},
			{"Bw1Bpp", null},
			{"R9G9B9E5SharedExponent", null},
			{"Rgbg8888", null},
			{"Grgb8888", null},
			{"Etc1", Etc1.Instance},
			{"Etc2Rgb", null},
			{"Etc2Rgba", null},
			{"EtcRgbA1", null},
			{"EacR11Unsigned", null},
			{"EacR11Signed", null},
			{"EacRg11Unsigned", null},
			{"EacRg11Signed", null},
		};

		public bool IsBasicFormat { get; private set; }
		public PvrBasicPixelFormat BasicFormat { get; private set; }
		public string ChannelOrder { get; private set; }
		public byte[] BitRates { get; private set; }

		public IColorFormat ColorFormat { get; private set; }

		public void Parse(BinaryReader reader)
		{
			uint pixelFormat1 = reader.ReadUInt32();
			uint pixelFormat2 = reader.ReadUInt32();
			IsBasicFormat = pixelFormat2 == 0;
			if (IsBasicFormat == false)
			{
				ChannelOrder = new string(new[]
				{
					(char)(byte)pixelFormat1,
					(char)(byte)(pixelFormat1 >> 8),
					(char)(byte)(pixelFormat1 >> 16),
					(char)(byte)(pixelFormat1 >> 24)
				});
				ChannelOrder = ChannelOrder.Trim('\0');

				var bitRates = new List<byte>();
				for (int index = 0; index < 32; index += 8)
				{
					byte rate = (byte)(pixelFormat2 >> index);
					if (rate == 0)
						break;
					bitRates.Add(rate);
				}

				BitRates = bitRates.ToArray();
			}
			else
				BasicFormat = (PvrBasicPixelFormat)pixelFormat1;

			ColorFormat = ResolveColorFormat();
		}

		private IColorFormat ResolveColorFormat()
		{
			string name = ToString();
			return ColorLookup.ContainsKey(name) ? ColorLookup[name] : null;
		}

		public override string ToString()
		{
			if (IsBasicFormat)
				return BasicFormat.ToString();

			string result = ChannelOrder;
			for (int index = 0; index < BitRates.Length; index++)
				result += BitRates[index];

			return result;
		}
	}
}
