﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using PvrSharp;

namespace PvrSharpVisualTest
{
	public partial class Form1 : Form
	{
		private Bitmap currentImage;
		private PvrFile currentPvrImage;
		private string baseMediaPath;

		public Form1()
		{
			InitializeComponent();
			BackColor = Color.CornflowerBlue;
			AllowDrop = true;

			string assemblyDir = Path.GetDirectoryName(Assembly.GetExecutingAssembly().Location);
			baseMediaPath = Path.Combine(assemblyDir, "../../../../media/");
			string[] files = Directory.GetFiles(baseMediaPath, "*.pvr");
			foreach (string file in files)
				listBox1.Items.Add(Path.GetFileName(file));
		}

		protected override void OnPaintBackground(PaintEventArgs e)
		{
			base.OnPaintBackground(e);

			if (currentImage != null)
				e.Graphics.DrawImage(currentImage, 0, 0);
		}

		protected override void OnDragEnter(DragEventArgs drgevent)
		{
			drgevent.Effect = drgevent.Data.GetDataPresent(DataFormats.FileDrop) ? DragDropEffects.Copy : DragDropEffects.None;
		}

		protected override void OnDragDrop(DragEventArgs drgevent)
		{
			var data = (Array)drgevent.Data.GetData(DataFormats.FileDrop);
			string filepath = data.GetValue(0).ToString();
			UpdateFile(filepath);
		}

		private void UpdateFile(string filepath)
		{
			Stream stream = File.OpenRead(filepath);
			currentPvrImage = PvrFile.FromStream(stream);
			stream.Close();

			if (currentImage != null)
				currentImage.Dispose();

			byte[] colorData = currentPvrImage.GetColorData();
			if (currentPvrImage.PixelFormat.ColorFormat.HasAlpha)
			{
				currentImage = new Bitmap((int)currentPvrImage.Width, (int)currentPvrImage.Height, PixelFormat.Format32bppArgb);
				int index = 0;
				for (int y = 0; y < (int)currentPvrImage.Height; y++)
					for (int x = 0; x < (int)currentPvrImage.Width; x++)
					{
						byte r = colorData[index++];
						byte g = colorData[index++];
						byte b = colorData[index++];
						byte a = colorData[index++];
						currentImage.SetPixel(x, y, Color.FromArgb(a, r, g, b));
					}
			}
			else
			{
				currentImage = new Bitmap((int)currentPvrImage.Width, (int)currentPvrImage.Height, PixelFormat.Format24bppRgb);
				int index = 0;
				for (int y = 0; y < (int)currentPvrImage.Height; y++)
					for (int x = 0; x < (int)currentPvrImage.Width; x++)
						currentImage.SetPixel(x, y,
							Color.FromArgb(colorData[index++], colorData[index++], colorData[index++]));
			}

			Refresh();
		}

		private void ListBox1SelectedIndexChanged(object sender, EventArgs e)
		{
			if (listBox1.SelectedIndex == -1)
				return;

			string filepath = Path.Combine(baseMediaPath, listBox1.SelectedItem.ToString());
			UpdateFile(filepath);
		}
	}
}
